@@ -53,7 +53,8 @@ void SQLiteDatabase::NoSchemaCallback () throw (Xception)
 				 "("
 					"name varchar(64) not null primary key,"
 					"type char(1) not null,"
-					"tbl_name varchar(64) not null"
+					"tbl_name varchar(64) not null,"
+					"purpose varchar(256)"
 				 ");", now;
 
 		(*m_pSession) << "create table TRejected "
@@ -75,8 +76,8 @@ void SQLiteDatabase::NoSchemaCallback () throw (Xception)
 					"v varchar(1024) not null"
 				 ");", now;
 
-		(*m_pSession) << "insert into TCollections values ('*','n','C0');", now;
-		(*m_pSession) << "insert into TCollections values ('#','t','C1');", now;
+		(*m_pSession) << "insert into TCollections values ('*','n','C0',NULL);", now;
+		(*m_pSession) << "insert into TCollections values ('#','t','C1',NULL);", now;
 
 		transactor.Commit();
 	}
@@ -86,7 +87,22 @@ void SQLiteDatabase::NoSchemaCallback () throw (Xception)
 	}
 }
 
-void SQLiteDatabase::CreateCollectionInSchema (const string& name, char type, const string& tblname) throw (Xception)
+void SQLiteDatabase::NoPurposeColumnCallback () throw (Xception)
+{
+	if (m_verbose)
+		cout << "updating schema" << endl;
+
+	try
+	{
+		(*m_pSession) << "alter table TCollections add column purpose varchar(256);", now;
+	}
+	catch (...)
+	{
+		throw Xception("could not update schema");
+	}
+}
+
+void SQLiteDatabase::CreateCollectionInSchema (const string& name, char type, const string& tblname, const string& purpose) throw (Xception)
 {
 	try
 	{
@@ -103,9 +119,10 @@ void SQLiteDatabase::CreateCollectionInSchema (const string& name, char type, co
 		}
 		sql += ");";
 		(*m_pSession) << sql, now;
+
 		string t;
 		t = type;
-		(*m_pSession) << "insert into TCollections values (:n,:t,:tbl_name);", use(name), use(t), use(tblname), now;
+		(*m_pSession) << "insert into TCollections values (:n,:t,:tbl_name,:purpose);", use(name), use(t), use(tblname), use(purpose), now;
 
 		transactor.Commit();
 	}
@@ -207,4 +224,16 @@ void SQLiteDatabase::InsertOrUpdateCollectionItem (const CollectionMetaInfo& cmi
 	}
 }
 
+void SQLiteDatabase::UpdatePurpose (const string& name, const string& purpose) throw (Xception)
+{
+	try
+	{
+		(*m_pSession) << "update TCollections set purpose=:p where name=:n;", use(purpose), use(name), now;
+	}
+	catch (...)
+	{
+		throw Xception("could not update collection purpose");
+	}
+}
+
 #endif // USE_SQLITE
