@@ -43,7 +43,7 @@ class ConfigImpl: public Config {
 ConfigImpl::ConfigImpl ()
 	: Config()
 {
-	m_od1.add_options()
+	m_od_visible.add_options()
 
 		// common options
 		("help,?",						"show this help screen and quit")
@@ -73,7 +73,7 @@ ConfigImpl::ConfigImpl ()
 
 	// in pdr we need a second options description for a hidden
 	// parameter containing all the command line arguments
-	m_od2.add_options() ("cmd_line_args", value<string>(), "");
+	m_od_cmdline.add_options() ("cmd_line_args", value<string>(), "");
 	m_pod.add("cmd_line_args", -1); // unlimited
 }
 
@@ -146,7 +146,7 @@ int main (int argc, char* argv[])
 		if (config->GetBoolOption("version"))
 		{
 			cout	<< "pdr - personal data recorder, " << VERSION << endl
-				<< "(C) 2010 Torsten Mueller, Bern, Switzerland" << endl
+				<< "(C) 2010-2011 Torsten Mueller, Bern, Switzerland" << endl
 				<< '(' << BUILD << ')' << endl
 				<< "This program is free software under the terms of the GNU General Public License v2 or later." << endl;
 			return 0;
