@@ -23,13 +23,14 @@ using namespace boost::gregorian;
 using namespace boost::program_options;
 
 #include "../libpdrx/datatypes.h"
-#include "../libpdrx/config.h"
 #include "../libpdrx/encoding.h"
+#include "config_impl.h"
 #include "diagram_impl.h"
 
 //=== DiagramImpl ==========================================================
 DiagramImpl::DiagramImpl ()
-	: m_xmin(not_a_date_time)
+	: m_sundays((RGB)-1)
+	, m_xmin(not_a_date_time)
 	, m_xmax(not_a_date_time)
 	, m_xunit((XUnit)-1)
 	, m_ymin(0.0)
@@ -39,7 +40,12 @@ DiagramImpl::DiagramImpl ()
 {
 }
 
-void DiagramImpl::DrawAxes (const ptime& x_min, const ptime& x_max, XUnit x_unit, double y_min, double y_max, double y_unit, RGB color)
+void DiagramImpl::SetSundaysColor (RGB color)
+{
+	m_sundays = color;
+}
+
+void DiagramImpl::DrawAxes (const ptime& x_min, const ptime& x_max, XUnit x_unit, double y_min, double y_max, double y_unit, RGB color, const ConfigImpl::Periods& /*periods*/)
 {
 	m_xmin = x_min;
 	m_xmax = x_max;
@@ -119,25 +125,6 @@ void DiagramImpl::DrawBars (const Selection& s, RGB color, int bar1, int bar2, c
 	if (pDimensions[0] == 1 && any_cast<string>(s[0][0]).empty())
 		throw Xception("selection contains just one single value which is the result of an aggregation, you must ensure the selection to have multiple values to draw bars");
 
-	struct Dot {
-		ptime	m_timestamp;
-		any	m_value;
-		Dot ()
-			: m_timestamp(not_a_date_time)
-			, m_value()
-		{
-		}
-		Dot (const string& timestamp, const any& value)
-			: m_timestamp(Diagram::CompletePtimeFromString(timestamp))
-			, m_value(value)
-		{
-		}
-		operator bool () const
-		{
-			return !m_value.empty();
-		}
-	};
-
 	for (size_t row = 0; row < pDimensions[0]; row++)
 	{
 		Dot current(any_cast<string>(s[row][0]), s[row][1]);
