@@ -24,8 +24,8 @@ using namespace boost::program_options;
 using namespace boost::filesystem;
 
 #include "../libpdrx/datatypes.h"
-#include "../libpdrx/config.h"
 #include "../libpdrx/encoding.h"
+#include "config_impl.h"
 #include "diagram_impl.h"
 
 #ifdef USE_CAIRO
@@ -45,8 +45,8 @@ using namespace boost::filesystem;
 #define	GREEN(c)	((double)(unsigned char)(c >>  8)) / 255.0
 #define	BLUE(c)		((double)(unsigned char)(c      )) / 255.0
 
-//=== DiagramCairo =========================================================
-DiagramCairo::DiagramCairo (bool antialias)
+//=== CairoDiagram =========================================================
+CairoDiagram::CairoDiagram (bool antialias)
 	: DiagramImpl()
 	, m_pSurface(NULL)
 	, m_pContext(NULL)
@@ -59,7 +59,7 @@ DiagramCairo::DiagramCairo (bool antialias)
 	m_margin.left = m_margin.top = m_margin.right = m_margin.bottom = 0.0;
 }
 
-DiagramCairo::~DiagramCairo ()
+CairoDiagram::~CairoDiagram ()
 {
 	if (m_pContext)
 		cairo_destroy(m_pContext);
@@ -67,7 +67,7 @@ DiagramCairo::~DiagramCairo ()
 		cairo_surface_destroy(m_pSurface);
 }
 
-void DiagramCairo::OpenCanvas (size_t width, size_t height, RGB bgColor)
+void CairoDiagram::OpenCanvas (size_t width, size_t height, RGB bgColor)
 {
 	m_diagram.width = (int)width;
 	m_diagram.height = (int)height;
@@ -96,8 +96,8 @@ void DiagramCairo::OpenCanvas (size_t width, size_t height, RGB bgColor)
 	}
 
 	// set the canvas size
-	m_canvas.width = (int)width + m_margin.left + m_margin.right;
-	m_canvas.height = (int)height + m_margin.top + m_margin.bottom;
+	m_canvas.width = int(width) + int(m_margin.left + m_margin.right);
+	m_canvas.height = int(height) + int(m_margin.top + m_margin.bottom);
 
 	// initially we put the origin into the upper right corner of
 	// the canvas
@@ -129,7 +129,7 @@ void DiagramCairo::OpenCanvas (size_t width, size_t height, RGB bgColor)
 	cairo_new_path(m_pContext);
 }
 
-void DiagramCairo::CloseCanvas ()
+void CairoDiagram::CloseCanvas ()
 {
 	// now we create a resulting surface with the real size
 	cairo_surface_t* pSurface = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, m_canvas.width - (int)m_origin.x + 1, (int)m_origin.y + 1);
@@ -155,9 +155,9 @@ void DiagramCairo::CloseCanvas ()
 	m_pSurface = pSurface;
 }
 
-void DiagramCairo::DrawAxes (const ptime& x_min, const ptime& x_max, XUnit x_unit, double y_min, double y_max, double y_unit, RGB color)
+void CairoDiagram::DrawAxes (const ptime& x_min, const ptime& x_max, XUnit x_unit, double y_min, double y_max, double y_unit, RGB color, const ConfigImpl::Periods& periods)
 {
-	DiagramImpl::DrawAxes(x_min, x_max, x_unit, y_min, y_max, y_unit, color);
+	DiagramImpl::DrawAxes(x_min, x_max, x_unit, y_min, y_max, y_unit, color, periods);
 
 	const double x_0 = CalcX(m_xmin);
 	const double y_0 = CalcY(m_ymin);
@@ -168,7 +168,46 @@ void DiagramCairo::DrawAxes (const ptime& x_min, const ptime& x_max, XUnit x_uni
 	cairo_fill(m_pContext);
 	UpdateOrigin(x_0, y_0);
 
-	// Step 1: x-axis
+	// Step 1: iterate over the time periods, make backgrounds
+	foreach(const ConfigImpl::Period* pPeriod, periods)
+	{
+		ptime begin(not_a_date_time), end(not_a_date_time);
+
+		{
+			ptime t(pPeriod->m_begin, time_duration(0, 0, 0, 0));
+			if (t < x_min)
+				begin = x_min;
+			else
+			{
+				if (t > x_max)
+					begin = x_max;
+				else
+					begin = t;
+			}
+		}
+
+		{
+			ptime t(pPeriod->m_end, time_duration(0, 0, 0, 0));
+			if (t < x_min)
+				end = x_min;
+			else
+			{
+				if (t > x_max)
+					end = x_max;
+				else
+					end = t;
+			}
+		}
+
+		if (begin != end)
+		{
+			cairo_set_source_rgb(m_pContext, RED(pPeriod->m_color), GREEN(pPeriod->m_color), BLUE(pPeriod->m_color));
+			cairo_rectangle(m_pContext, CalcX(begin), CalcY(y_min), CalcX(end) - CalcX(begin), CalcY(y_max) - CalcY(y_min));
+			cairo_fill(m_pContext);
+		}
+	}
+
+	// Step 2: x-axis
 	cairo_set_source_rgb(m_pContext, RED(color), GREEN(color), BLUE(color));
 	cairo_set_line_width(m_pContext, m_thickness);
 	move_to(x_0, y_0);
@@ -219,12 +258,19 @@ void DiagramCairo::DrawAxes (const ptime& x_min, const ptime& x_max, XUnit x_uni
 				double x = CalcX(t);
 				move_to(x, y_0);
 				line_to(x, y_0 + dash_height);
-				text_x(x, y_0, dash_height, s2);
+				if (m_sundays != (RGB)-1 && m_xunit == days && t.date().day_of_week() == 0)
+				{
+					cairo_set_source_rgb(m_pContext, RED(m_sundays), GREEN(m_sundays), BLUE(m_sundays));
+					text_x(x, y_0, dash_height, s2);
+					cairo_set_source_rgb(m_pContext, RED(color), GREEN(color), BLUE(color));
+				}
+				else
+					text_x(x, y_0, dash_height, s2);
 			}
 		}
 	}
 
-	// Step 2: y-axis, much easier because of simple values
+	// Step 3: y-axis, much easier because of simple values
 	move_to(x_0, y_0);
 	line_to(x_0, y_0 - m_diagram.height);
 
@@ -244,7 +290,7 @@ void DiagramCairo::DrawAxes (const ptime& x_min, const ptime& x_max, XUnit x_uni
 	cairo_stroke(m_pContext);
 }
 
-void DiagramCairo::DrawAxes (Diagram::FoldInterval fi, double y_min, double y_max, double y_unit, RGB color)
+void CairoDiagram::DrawAxes (Diagram::FoldInterval fi, double y_min, double y_max, double y_unit, RGB color)
 {
 	DiagramImpl::DrawAxes(fi, y_min, y_max, y_unit, color);
 
@@ -317,32 +363,13 @@ void DiagramCairo::DrawAxes (Diagram::FoldInterval fi, double y_min, double y_ma
 	cairo_stroke(m_pContext);
 }
 
-void DiagramCairo::DrawCurve (const Selection& s, RGB color, LineStyle style, double thickness, bool continuous)
+void CairoDiagram::DrawCurve (const Selection& s, RGB color, LineStyle style, double thickness, bool continuous)
 {
 	const size_t* pDimensions = s.shape(); // [0] rows, [1] columns
 
 	if (pDimensions[0] == 1 && any_cast<string>(s[0][0]).empty())
 		throw Xception("selection contains just one single value which is the result of an aggregation, you must ensure the selection to have multiple values to draw a curve");
 
-	struct Dot {
-		ptime	m_timestamp;
-		any	m_value;
-		Dot ()
-			: m_timestamp(not_a_date_time)
-			, m_value()
-		{
-		}
-		Dot (const string& timestamp, const any& value)
-			: m_timestamp(Diagram::CompletePtimeFromString(timestamp))
-			, m_value(value)
-		{
-		}
-		operator bool () const
-		{
-			return !m_value.empty();
-		}
-	};
-
 	if (thickness == 0.0)
 		thickness = m_thickness;
 
@@ -355,6 +382,21 @@ void DiagramCairo::DrawCurve (const Selection& s, RGB color, LineStyle style, do
 
 		switch (style)
 		{
+			case symbol_dot:
+			{
+				if (current)
+				{
+					if (current.m_timestamp < m_xmin)
+						continue;
+					cairo_set_line_width(m_pContext, thickness);
+					double x = CalcX(current.m_timestamp);
+					double y = CalcY(any_cast<double>(current.m_value));
+					move_to(x - thickness / 2.0, y);
+					line_to(x + thickness / 2.0, y);
+					cairo_stroke(m_pContext);
+				}
+				break;
+			}
 			case symbol_plus:
 			{
 				if (current)
@@ -405,7 +447,25 @@ void DiagramCairo::DrawCurve (const Selection& s, RGB color, LineStyle style, do
 				}
 				break;
 			}
-			case symbol_cross:
+			case symbol_x:
+			{
+				if (current)
+				{
+					if (current.m_timestamp < m_xmin)
+						continue;
+					cairo_set_line_width(m_pContext, thickness);
+					double x = CalcX(current.m_timestamp);
+					double y = CalcY(any_cast<double>(current.m_value));
+					const double extent = m_diagram.width / 200.0; // half a percent of m_diagram.width
+					move_to(x - extent, y - extent);
+					line_to(x + extent, y + extent);
+					move_to(x - extent, y + extent);
+					line_to(x + extent, y - extent);
+					cairo_stroke(m_pContext);
+				}
+				break;
+			}
+			case symbol_X:
 			{
 				if (current)
 				{
@@ -439,6 +499,22 @@ void DiagramCairo::DrawCurve (const Selection& s, RGB color, LineStyle style, do
 				}
 				break;
 			}
+			case symbol_square:
+			{
+				if (current)
+				{
+					if (current.m_timestamp < m_xmin)
+						continue;
+					cairo_set_line_width(m_pContext, thickness);
+					double x = CalcX(current.m_timestamp);
+					double y = CalcY(any_cast<double>(current.m_value));
+					const double extent = m_diagram.width / 100.0; // one percent of m_diagram.width
+					cairo_rectangle(m_pContext, x - extent / 2.0, y - extent / 2.0, extent, extent);
+					UpdateOrigin(x - extent / 2.0, y + extent / 2.0);
+					cairo_stroke(m_pContext);
+				}
+				break;
+			}
 			default: // zick-zack
 			{
 				if (current.m_timestamp < m_xmin)
@@ -459,7 +535,40 @@ void DiagramCairo::DrawCurve (const Selection& s, RGB color, LineStyle style, do
 	}
 }
 
-void DiagramCairo::DrawVLine (const ptime& timestamp, RGB color, double thickness)
+void CairoDiagram::DrawStairs (const Selection& s, RGB color, double thickness)
+{
+	const size_t* pDimensions = s.shape(); // [0] rows, [1] columns
+
+	if (pDimensions[0] == 1 && any_cast<string>(s[0][0]).empty())
+		throw Xception("selection contains just one single value which is the result of an aggregation, you must ensure the selection to have multiple values to draw a curve");
+
+	if (thickness == 0.0)
+		thickness = m_thickness;
+
+	cairo_set_source_rgb(m_pContext, RED(color), GREEN(color), BLUE(color));
+
+	Dot last;
+	for (size_t row = 0; row < pDimensions[0]; row++)
+	{
+		Dot current(any_cast<string>(s[row][0]), s[row][1]);
+
+		if (current.m_timestamp < m_xmin)
+			continue;
+		if (last && current)
+		{
+			cairo_set_line_width(m_pContext, thickness);
+			move_to(CalcX(last.m_timestamp), CalcY(any_cast<double>(last.m_value)));
+			line_to(CalcX(current.m_timestamp), CalcY(any_cast<double>(last.m_value)));
+			line_to(CalcX(current.m_timestamp), CalcY(any_cast<double>(current.m_value)));
+			cairo_stroke(m_pContext);
+		}
+
+		if (current)
+			last = current;
+	}
+}
+
+void CairoDiagram::DrawVLine (const ptime& timestamp, RGB color, double thickness)
 {
 	cairo_set_source_rgb(m_pContext, RED(color), GREEN(color), BLUE(color));
 	cairo_set_line_width(m_pContext, (float)((thickness == 0.0) ? m_thickness : thickness));
@@ -470,12 +579,12 @@ void DiagramCairo::DrawVLine (const ptime& timestamp, RGB color, double thicknes
 	cairo_stroke(m_pContext);
 }
 
-void DiagramCairo::DrawVLine (const time_duration& time, RGB color, double thickness)
+void CairoDiagram::DrawVLine (const time_duration& time, RGB color, double thickness)
 {
 	DrawVLine(ptime(date(9999, 1, 1), time), color, thickness);
 }
 
-void DiagramCairo::DrawHLine (double value, RGB color, double thickness)
+void CairoDiagram::DrawHLine (double value, RGB color, double thickness)
 {
 	cairo_set_source_rgb(m_pContext, RED(color), GREEN(color), BLUE(color));
 	cairo_set_line_width(m_pContext, (float)((thickness == 0.0) ? m_thickness : thickness));
@@ -486,14 +595,7 @@ void DiagramCairo::DrawHLine (double value, RGB color, double thickness)
 	cairo_stroke(m_pContext);
 }
 
-void DiagramCairo::SaveAs (const string& filename) const
-{
-	cairo_status_t st = cairo_surface_write_to_png(m_pSurface, filename.c_str());
-	if (st != CAIRO_STATUS_SUCCESS)
-		throw Xception(cairo_status_to_string(st));
-}
-
-void DiagramCairo::DrawBar (const ptime& t, double value, int bar1, int bar2, RGB color, bool fix)
+void CairoDiagram::DrawBar (const ptime& t, double value, int bar1, int bar2, RGB color, bool fix)
 {
 	double x = CalcX(t);
 	double y = CalcY(value);
@@ -510,7 +612,7 @@ void DiagramCairo::DrawBar (const ptime& t, double value, int bar1, int bar2, RG
 	}
 }
 
-void DiagramCairo::UpdateOrigin (double x, double y)
+void CairoDiagram::UpdateOrigin (double x, double y)
 {
 	if (x < m_origin.x)
 		m_origin.x = x;
@@ -518,33 +620,33 @@ void DiagramCairo::UpdateOrigin (double x, double y)
 		m_origin.y = y;
 }
 
-double DiagramCairo::CalcX (const ptime& x) const
+double CairoDiagram::CalcX (const ptime& x) const
 {
 	double a = (x - m_xmin).total_seconds();
 	double b = (m_xmax - m_xmin).total_seconds();
 	return (a / b) * m_diagram.width + m_margin.left;
 }
 
-double DiagramCairo::CalcY (double y) const
+double CairoDiagram::CalcY (double y) const
 {
 	double a = y - m_ymin;
 	double b = m_ymax - m_ymin;
 	return m_diagram.height - (a / b) * m_diagram.height + m_margin.top;
 }
 
-void DiagramCairo::move_to (double x, double y)
+void CairoDiagram::move_to (double x, double y)
 {
 	cairo_move_to(m_pContext, x, y);
 	UpdateOrigin(x, y);
 }
 
-void DiagramCairo::line_to (double x, double y)
+void CairoDiagram::line_to (double x, double y)
 {
 	cairo_line_to(m_pContext, x, y);
 	UpdateOrigin(x, y);
 }
 
-void DiagramCairo::text_x (double x, double y, double dash_height, const string& s)
+void CairoDiagram::text_x (double x, double y, double dash_height, const string& s)
 {
 	cairo_save(m_pContext);
 
@@ -557,7 +659,7 @@ void DiagramCairo::text_x (double x, double y, double dash_height, const string&
 	cairo_restore(m_pContext);
 }
 
-void DiagramCairo::text_y (double x, double y, double dash_width, const string& s)
+void CairoDiagram::text_y (double x, double y, double dash_width, const string& s)
 {
 	cairo_text_extents_t extents;
 	cairo_text_extents(m_pContext, s.c_str(), &extents);
@@ -565,4 +667,17 @@ void DiagramCairo::text_y (double x, double y, double dash_width, const string&
 	cairo_show_text(m_pContext, s.c_str());
 }
 
+//=== DiagramPNG ===========================================================
+DiagramPNG::DiagramPNG (bool antialias)
+	: CairoDiagram(antialias)
+{
+}
+
+void DiagramPNG::SaveAs (const string& filename) const
+{
+	cairo_status_t st = cairo_surface_write_to_png(m_pSurface, filename.c_str());
+	if (st != CAIRO_STATUS_SUCCESS)
+		throw Xception(cairo_status_to_string(st));
+}
+
 #endif // USE_CAIRO
